// Christian Duncan
// Node Class
//    for GD2004, Graph Drawing Contest

// package GD4;

import java.util.Vector;
import java.util.Iterator;

/*********************
 * A Node in the Graph
 *    Abstract Class
 *    Stores:
 *       id - integer index into Graph Vector list (currently irrelevant!)
 *       X, Y - double location coordinates
 *********************/
public abstract class Node {
    protected int id;
    protected double x, y;
    protected boolean pinState = false; // True if "pinned down"

    public Node(int _id, double _x, double _y) {
	id = _id; x = _x; y = _y;
    }

    public Node(Node other) {
	id = other.id; x = other.x; y = other.y;
	pinState = other.pinState;
    }

    /* Accessor methods */
    public int getId() { return id; }
    public double getX() { return x; }
    public double getY() { return y; }
    public abstract Iterator iterator(); // Return iterator of incident edges
    public boolean isPinned() { return pinState; }

    /* Mutator methods */
    // public void setId(int _id) { id = _id; }
    public void setX(double _x) { x = _x; }
    public void setY(double _y) { y = _y; }
    public void flipPinState() { pinState = !pinState; }
    public void setPinned(boolean newState) { pinState = newState; }

    /* Add an edge to the list of edges
     */
    abstract void addEdge(Node other);

    public double distance(Node other) {
	return Math.sqrt(sqrDistance(other));
    }

    public double sqrDistance(Node other) {
	double dx = other.x - this.x;
	double dy = other.y - this.y;
	return dx * dx + dy * dy;
    }

    // Two nodes are equal if they have the same ID!
    public boolean equals(Node other) {
	return this.id == other.id;
    }
}
